#!/usr/bin/env python

'''

    @version $Id: acme,v 1.6 2005/09/17 01:31:47 dave Exp $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import pygame
from pygame.locals import *
import libwidget

eventMaster = None

class eventManager(object):
    eventList = None
    
    def __init__(self):
        #from weakref import WeakKeyDictionary
        #self.listeners = WeakKeyDictionary()
        self.listeners = {}

        self.eventList = []

    # evtType should be a tuple containing (evtType, Widget)
    # pass it None for the source if you don't have a source for it
    def RegisterListener( self, source, callback, evtType ):
        self.listeners[ callback ] = (evtType, source)
    
    def UnregisterListener( self, callback ):
        if listener in self.listeners.keys():
            del self.listeners[ listener ]

    def Poll(self):
        for event in pygame.event.get():
            if event.type == NOEVENT:
                break
            if event.type == KEYDOWN:
                # Let's go ahead and get a key for the thing
                self.Post(libwidget.mgKeyboardInput(character=event.unicode, key=event.key, modifiers=event.mod, source=None) )
            if event.type == MOUSEBUTTONUP:
                if event.button == 1:
                    self.Post(libwidget.mgMouseClick(pos=event.pos, source=None ) )
                if event.button == 2:
                    pass
                if event.button == 3:
                    self.Post(libwidget.mgRightClick(pos=event.pos, source=None) )
            if event.type == MOUSEMOTION:
                self.Post(libwidget.mgMouseMove(pos=event.pos, source=None ) )
            if event.type == QUIT:
                self.Post(libwidget.mgQuit(source=None))

    def ProcessQueue(self):
        # This is possibly the most inefficient way possible to do this
        for event in self.eventList:
            for listener in self.listeners.keys():
                #NOTE: If the weakref has died, it will be
                #automatically removed, so we don't have
                #to worry about it.
                if self.listeners[listener][0] == event.mgEType and event.sourcewidget == self.listeners[listener][1]:
                    listener( event )
        self.eventList = []
    
    def Post( self, event ):
        self.eventList.append(event)

