#!/usr/bin/env python

'''

    @version $Id: font.py,v 1.6 2005/09/19 06:21:54 dave Exp $

    This file is part of mathguy.

    mathguy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    mathguy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mathguy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    mathguy is copyright 2005 by David Fancella

'''

import pygame
from pygame.locals import *
from libacme import paths
from libGUIacme import graphics

class mgFont(object):
    thefont = None
    module = None
    myRect = None
    
    def __init__(self, fontfile=None, **args):
        self.size = 36

        if fontfile is None:
            fontfile = "Imperium"
            
        if args.has_key("size"):
            self.size = args['size']
        if args.has_key("module"):
            self.module = args['module']
        if args.has_key("shadow_color"):
            self.shadowColor = args['shadow_color']
        else:
            self.shadowColor = (0,0,0)
    
        self.thefont = paths.LoadFont(fontfile, self.size, self.module)

    def get_linesize(self):
        return self.thefont.get_linesize()
        
    # text is the string to print
    # color is a RGB triplet, tuple
    # returns a surface containing the text
    def fprint(self, text, color, width=None):
        theText = []
        text = text.strip()
        # If we need to make it fit within a certain width, do it here
        if (width is not None) and (width > 0):
            Oldtext = text.split("\n")
            
            for text in Oldtext:
                start = 0
                end = len(text) + 1
    
                stop = False
                # start the loop here
                if self.thefont.size(text[start:end])[0] < width:
                    theText.append(text)
                else:
                    while not stop:
                        if self.thefont.size(text[start:end])[0] < width:
                            theText.append(text[start:end])
                            break
                        end = text.rfind(' ', start, end)
                        
                        if end == -1:
                            # no more spaces left in the string, get what's left and stop
                            theText.append(text[start:len(text)])
                            stop = True
                        elif self.thefont.size(text[start:end])[0] < width:
                            # still more spaces, maybe, get this chunk and keep going
                            theText.append(text[start:end])
                            start = end + 1
                            end = len(text)
                        if start >= end:
                            break
                # Loop's over
            # So is this one
        else:
            theText.append(text)
        # now render the string with shadow and stuff
        surfList = []
        spacing = int(self.size * 0.05)
        #spacing = 2
        shadow = 2
        # calculate and create the surface that will hold the final rendered
        # text
        # first find the widest line
        width = 0
        for line in theText:
            tmpWidth, tmpHeight = self.thefont.size(line)
            if tmpWidth > width:
                width = tmpWidth
        width += shadow
        self.width = width
        height = (self.size + spacing)
        self.height = height
        size = ((width), len(theText) * height + 5)
        tmpSurface = graphics.Surface(size)
        #tmpSurface.set_colorkey( (0,0,0) )
        linespacing = 0
        for line in range(len(theText)):
            if line > 0:
                linespacing = spacing
            # render and place the shadow first
            tmpSurface.blit(self.thefont.render(theText[line], 1, self.shadowColor),
                              ( shadow, (line * height) + shadow)  )
            # render and place the text
            tmpSurface.blit(self.thefont.render(theText[line], 1, color),
                              (0, (line * height))  )
        self.myRect = pygame.Rect( (0,0), ( (width), (len(theText) * height) ) )
        tmpSurface.convert()
        return tmpSurface

