#!/usr/bin/env python

'''

    @version $Id: acme,v 1.6 2005/09/17 01:31:47 dave Exp $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import pygame
from pygame.locals import *
from libGUIacme import event_manager
from libGUIacme.libwidget import *
from libGUIacme import graphics
from libacme import paths
from libacme import map_parser
import random, os
from libGUIacme.widgets.mg_constants import *

class mgGame(object):
    endgame = False
    games = []
    timetick = 0.0
    starttime = 0.0
    screen = None
    background = None
    fonts = {}
    title = None
    eventManager = None

    def __init__(self, starttime, screen, resolution):
        self.timetick = starttime
        self.starttime = starttime
        self.screen = screen
        self.backgroundImage = graphics.LoadImage( paths.Join('graphics', 'rim_wall.png') )
        self.background = graphics.TileImage(self.backgroundImage, resolution)
        self.screen.blit(self.background, [0, 0])
        event_manager.eventMaster = event_manager.eventManager()
        self.eventManager = event_manager.eventMaster
        self.widgets = pygame.sprite.RenderUpdates()
        
        self.mapView = mgMapView( pos=(3,3), size=(551, 551), eventmanager = self.eventManager )
        self.wallButton = mgToolButton( pos=(600,100), eventmanager = self.eventManager,
                                    icon = paths.Join('icons', 'walls.png'), toggle=True)
        self.spawnButton = mgToolButton( pos=(600,150), eventmanager = self.eventManager,
                                    icon = paths.Join('icons', 'spawn.png'), toggle=True)
        self.saveButton = mgButton( pos=(600,300), eventmanager = self.eventManager,
                                    caption = "Save", font_size = 20, toggle=False)
        self.metaPanel = mgMapMeta( pos=(600,400), eventmanager = self.eventManager)
        self.theMap = self.mapView.theMap
                                    
        self.widgets.add(self.mapView)
        self.widgets.add(self.wallButton)
        self.widgets.add(self.spawnButton)
        self.widgets.add(self.saveButton)
        self.widgets.add(self.metaPanel)

        self.eventManager.RegisterListener(self.wallButton, self.OnWallButtonClick, mgETOOLTOGGLE)
        self.eventManager.RegisterListener(self.spawnButton, self.OnSpawnButtonClick, mgETOOLTOGGLE)
        self.eventManager.RegisterListener(self.saveButton, self.OnSaveButtonClick, mgEBUTTONCLICK)
        self.eventManager.RegisterListener(None, self.OnQuit, mgEQUIT)

    def LoadMap(self, mapfile):
        self.theMapFile = mapfile
        self.theMap = map_parser.ParseMap(mapfile)
        self.mapView.SetMap(self.theMap)
        self.metaPanel.authorField.SetText(self.theMap.mapAuthor)
        self.metaPanel.mapVersionField.SetText(self.theMap.mapVersion)
        self.metaPanel.mapNameField.SetText(self.theMap.mapName)
        
        
    def loop(self):
        frames = 0
        self.quitProgram = False
        self.starttime = self.getTime()

        while not self.quitProgram:
            # The main menu goes here
            ticknow = self.getTime()
            frames += 1
            
            self.eventManager.Poll()
            self.eventManager.ProcessQueue()
            
            # This clear call is *supposed* to clear the background quickly
            # but doesn't appear to be doing so, hence the fill call
            # todo: fix this
            self.widgets.clear(self.screen, self.background)
            #self.screen.fill( (254,254,254) )
            self.widgets.update(ticknow)
            for b in self.widgets.sprites():
                b.UpdateView(self.screen)
            pygame.display.flip()
        
        print "fps: %.3f" % ( (float(frames) / (ticknow - self.starttime)) )
        print "%d frames in %.2f seconds" % (frames, (float(ticknow - self.starttime)) )

    def OnQuit(self, event):
        self.quitProgram = True
        
    def OnWallButtonClick(self, event):
        if event.sourcewidget.isToggled:
            self.mapView.StartWall()
        else:
            self.mapView.FinishWall()

    def OnSpawnButtonClick(self, event):
        if event.sourcewidget.isToggled:
            self.mapView.StartSpawn()
        else:
            self.mapView.FinishSpawn()

    def OnSaveButtonClick(self, event):
        self.theMap.SetName(self.metaPanel.mapNameField.GetText())
        self.theMap.SetAuthor(self.metaPanel.authorField.GetText())
        self.theMap.SetVersion(self.metaPanel.mapVersionField.GetText())
        self.theMap.SaveMap(self.theMapFile)
    
    def getTime(self):
        return float(pygame.time.get_ticks()/1000.0)
                
    def updateGame(self, timetick):
        self.currentGame.updateGame(timetick)

    def updateView(self, timetick):
        self.currentGame.updateView(timetick)
        
    def processKey(self, key, timetick):        
        if key==K_f:
            pygame.display.toggle_fullscreen()
        else:
            self.currentGame.processKey(key, timetick)
            
    

