#!/usr/bin/env python

'''

    @version $Id: mg_event.py,v 1.2 2005/09/27 04:40:44 dave Exp $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import pygame
from pygame.locals import *
from libGUIacme.widgets.mg_constants import *

class mgEvent(object):
    mgEType = mgEBASEEVENT
    mgBaseType = mgEBASEEVENT
    timetick = 0.0
    active = None
    sourcewidget = None

    def __init__(self, **args):
        if args.has_key("type"):
            self.mgEType = args['type']
        if args.has_key("timetick"):
            self.timetick = args['timetick']
        if args.has_key("source"):
            self.sourcewidget = args['source']
        else:
            raise RuntimeError, 'Event requires a source!'

        self.active = True

    def KillEvent(self):
        self.active = False

    def skip(self):
        pass

class mgQuit(mgEvent):
    mgEType = mgEQUIT

# base class for input events
class mgInputEvent(mgEvent):
    mgBaseType = mgEINPUTEVENT

    def __init__(self, **args):
        super(mgInputEvent, self).__init__(**args)
    
# This is the event that stores all keyboard input
# You can access pieces of it depending on how it's constructed
class mgKeyboardInput(mgInputEvent):
    mgEType = mgEKEYBOARD
    key = None
    modifiers = None
    keyname = None
    character = None
    
    def __init__(self, **args):
        super(mgKeyboardInput, self).__init__(type=self.mgEType, **args)

        if args.has_key("key"):
            self.key = args['key']
        else:
            raise RuntimeError, "You can't create a keyboard event without a key!"
            
        if args.has_key("character"):
            self.character = args['character']
        else:
            raise RuntimeError, "You can't create a keyboard event without a character!"
        
        if args.has_key("modifiers"):
            self.modifiers = args["modifiers"]

        self.keyname = pygame.key.name(self.key)
        
        
            
# you can override many of the mgMouseClick attributes by just passing them in as
# attribute=value.  If you don't, pygame will be queried directly, which is normally
# the behavior you want anyway
class mgMouseClick(mgInputEvent):
    pos = None
    mgEType = mgEMOUSECLICK
    buttons = {}
    
    def __init__(self, **args):
        super(mgMouseClick, self).__init__(**args)

        if args.has_key("pos"):
            self.pos = args['pos']
        else:
            self.pos = pygame.mouse.get_pos()

class mgRightClick(mgMouseClick):
    mgEType = mgERIGHTCLICK

class mgMouseMove(mgInputEvent):
    pos = None
    mgEType = mgEMOUSEMOVE
    buttons = {}
    
    def __init__(self, **args):
        super(mgMouseMove, self).__init__(**args)

        if args.has_key("pos"):
            self.pos = args['pos']
        else:
            self.pos = pygame.mouse.get_pos()

        if args.has_key("buttons"):
            self.buttons["left"] = args["buttons"][0]
            self.buttons["middle"] = args["buttons"][1]
            self.buttons["right"] = args["buttons"][2]
        else:
            buttons = pygame.mouse.get_pressed()
            self.buttons["left"] = buttons[0]
            self.buttons["middle"] = buttons[1]
            self.buttons["right"] = buttons[2]


