#!/usr/bin/env python

'''

    @version $Id: mg_event.py,v 1.2 2005/09/27 04:40:44 dave Exp $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import pygame
from pygame.locals import *
from libGUIacme import font
from libacme import paths
from libGUIacme.libwidget import *
from libGUIacme.widgets import mg_event
from libGUIacme import graphics
from libGUIacme.widgets import mg_widget
from libGUIacme.widgets.mg_constants import *

from libacme.map import aaMap

import math

# This class represents a view of a loaded map
class mgMapView(mg_widget.mgWidgetBase):
    theMap = None
    gridXYSize = (570, 570)
    origin = (20,520)
    mode = None
    data = None
    gridSnap = True
    gridSnapBy = 10
    
    def __init__(self, **args):
        super(mgMapView, self).__init__(**args)

        if args.has_key('map'):
            self.theMap = args['map']
        else:
            self.theMap = aaMap()
        if args.has_key('gridsize'):
            self.gridSize = args['gridsize']
        else:
            self.gridSize = 10.0
            
        self.background = graphics.Surface( self.size )
        self.background.fill( (255, 255, 255, 255) )
        self.background.convert()

        self.RegisterHandler(mgEMOUSECLICK, self.OnMouseClick, None)
        self.RegisterHandler(mgEMOUSEMOVE, self.OnMouseMove, None)
        self.RegisterHandler(mgERIGHTCLICK, self.OnRightClick, None)

    def OnMouseClick(self, event):
        if self.rect.collidepoint(event.pos):
            if self.mode == "wall":
                self.data.append(self.GridSnap(event.pos) )
                self._drawNewWalls()
            elif self.mode == 'spawn':
                self.theMap.AddSpawn( pos=self.OrderPointFromDisplay(self.GridSnap(event.pos)),
                                      vector=(1,0),
                                      angle=0 )

    def OnRightClick(self, event):
        if self.rect.collidepoint(event.pos):
            if self.mode == 'wall':
                self.data.append(self.GridSnap(event.pos) )
                self.FinishWall()
                self.StartWall()

    def GridSnap(self, point):                
        if self.gridSnap:
            newPosX, newPosY = point
            newPosX -= newPosX % self.gridSnapBy
            newPosY -= newPosY % self.gridSnapBy
            return (newPosX, newPosY)
        else:
            return point

                                      
    def OnMouseMove(self, event):
        if self.mode == 'wall':
            self.currentWall = graphics.Surface( (551, 551) )
            if len(self.data) > 0:
                pygame.draw.line( self.currentWall, (100,70,50,100),
                                    self.data[len(self.data) - 1],
                                    self.GridSnap(event.pos),
                                    1)

    def StartWall(self):
        self.mode = 'wall'
        self.data = []
        self.newWalls = graphics.Surface( (551, 551) )
        self.currentWall = graphics.Surface( (551, 551) )

    def FinishWall(self):
        self.mode = None
        for a in range(len(self.data)):
            self.data[a] = self.OrderPointFromDisplay(self.data[a])
        if len(self.data) > 1:
            self.theMap.AddWall(points=self.data)
        self.data = None
        self.isDirty = True

    def StartSpawn(self):
        self.mode = 'spawn'

    def FinishSpawn(self):
        self.mode = None
            
    def SetMap(self, themap):
        self.theMap = themap

    def OrderPoint(self, point):
        x, y = point
        return int(x) + self.origin[0],  self.origin[1] - int(y)
        
    def OrderPointFromDisplay(self, point):
        x, y = point
        return int(x) - self.origin[0],  self.origin[1] - int(y)

    def DrawBackground(self):
        self.image.blit(self.background, (0,0) )
        
    def DrawGrid(self):
        # Draw the axes next
        # The X axis
        x, y = self.OrderPoint( (-540, 0) )
        x1, y1 = self.OrderPoint( (540, 0) )
        pygame.draw.line(self.image, ( 0, 255, 0 ),
                            (x, y), (x1, y1), 3 )
        # the Y axis
        x, y = self.OrderPoint( (0, -540) )
        x1, y1 = self.OrderPoint( (0, 540) )
        pygame.draw.line(self.image, ( 0, 255, 0 ),
                            (x, y), (x1, y1), 3 )
        
        # Draw the original square so mapmakers can build to scale
        p1 = self.OrderPoint( (1,1) )
        p2 = self.OrderPoint( (1,499) )
        p3 = self.OrderPoint( (499,499) )
        p4 = self.OrderPoint( (499,1) )
        pygame.draw.lines( self.image, (255,90,90, 200),
                              True, [p1, p2, p3, p4], 1)
        # Draw the grid
        for a in range(-540, 541, 20):
            x, y = self.OrderPoint( (a, -540) )
            x1, y1 = self.OrderPoint( (a, 540) )
            pygame.draw.line(self.image, ( 0, 255, 0 ),
                             (x, y), (x1, y1) )
            x, y = self.OrderPoint( (-540, a) )
            x1, y1 = self.OrderPoint( (540, a) )
            pygame.draw.line(self.image, ( 0, 255, 0 ),
                             (x, y), (x1, y1) )

    def _drawNewWalls(self):
        points = []
        for wall in self.data:
            points.append( wall )
            if len(points) > 1:
                pygame.draw.lines(self.newWalls, (0,150,20),
                                False, points, 1)
                             
    def DrawWalls(self):
        if len(self.theMap.walls) > 0:
            for wall in self.theMap.walls:
                points = []
                for a in range(len(wall)):
                    points.append( self.OrderPoint(wall[a]) )
                if len(points) > 1:
                    pygame.draw.lines(self.image, (0,0,0),
                                    False, points, 1)
        if len(self.theMap.obwalls) > 0:
            for wall in self.theMap.obwalls:
                points = []
                for a in range(len(wall)):
                    x,y = wall[a]
                    points.append( self.OrderPoint(wall[a]) )
                if len(points) > 1:
                    pygame.draw.lines(self.image, (0,0,0),
                                    False, points, 1)

    def DrawSpawnPoints(self):
        for a in self.theMap.spawnPoints:
            thePoint = self.OrderPoint(a['pos'])
            # top* should be the tip in the direction the point faces
            topX = thePoint[0] + (math.sin(a['angle']) * 3 )
            topY = thePoint[1] + (math.cos(a['angle']) * 3 )
            botX = thePoint[0] - (math.sin(a['angle']) * 1 )
            botY = thePoint[1] - (math.cos(a['angle']) * 1 )
            pygame.draw.line(self.image, (255,0,0), (topX, topY),
                                                    (botX, botY), 3 )

            # so we'll draw the arrow there
            botX = thePoint[0] - (math.sin(a['angle']) * 4 )
            botY = thePoint[1] - (math.cos(a['angle']) * 4 )
            topLX = botX - (math.sin( math.pi + a['angle'] + math.pi/4) * 3)
            topLY = botY - (math.cos( math.pi + a['angle'] + math.pi/4) * 3)
            topRX = botX - (math.sin( math.pi + a['angle'] - math.pi/4) * 3)
            topRY = botY - (math.cos( math.pi + a['angle'] - math.pi/4) * 3)
            pygame.draw.polygon(self.image, (255,0,0), [(topLX, topLY), (botX, botY), (topRX, topRY)])

    def update(self, timetick):
        if self.theMap.isDirty:
            self.DrawBackground()
            self.DrawGrid()
            self.DrawWalls()
            self.DrawSpawnPoints()
            self.DrawDecoration()
            self.theMap.isDirty = False

    def UpdateView(self, screen):
        screen.blit(self.image, self.rect)
        if self.mode == 'wall':
            screen.blit(self.newWalls, self.rect)
            screen.blit(self.currentWall, self.rect)


