#!/usr/bin/env python

'''

    @version $Id: mg_menu.py,v 1.7 2005/10/06 06:30:43 dave Exp $

    This file is part of mathguy.

    mathguy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    mathguy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mathguy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    mathguy is copyright 2005 by David Fancella

'''

import pygame
from pygame.locals import *
from libGUIacme import font
from libacme import paths
from libGUIacme.libwidget import *
from libGUIacme.widgets import mg_event
from libGUIacme import graphics
from libGUIacme.widgets import mg_widget
from libGUIacme.widgets.mg_constants import *

# This class represents a selection event
class evtSelection(mg_event.mgEvent):
    mgEType = mgESELECTION
    selection = None

    def __init__(self, **args):
        super(evtSelection, self).__init__(type=self.mgEType, **args)

        if args.has_key('selection'):
            self.selection = args['selection']
        else:
            raise RuntimeError, 'Selection Event requires a selection!'

class evtQuit(mg_event.mgEvent):
    mgEType = mgEQUIT

# This class represents a menu
class mgMenu(mg_widget.mgWidgetBase):
    choices = {}
    choiceLabels = {}
    currentSelection = {}
    topRef = None
    posRect = None
    
    def __init__(self, **args):
        super(mgMenu, self).__init__(**args)
        self.currentSelection[ str(self.widgetCount) ] = None
        
        self.EmptyMenu()
        
        if args.has_key("top_reference"):
            self.topRef = args['top_reference']
        else:
            self.topRef = None
        self.masterSurf = graphics.Surface(self.rasterSize)
        if self.backgroundColor is None:
            self.backgroundColor = 0x00000000

        self.RegisterHandler(mgEKEYBOARD, self.OnKeyboard)
        self.RegisterHandler(mgEMOUSEMOVE, self.OnMouseMove)
        self.RegisterHandler(mgEMOUSECLICK, self.OnMouseClick)

    def OnKeyboard(self, event):
        if event.key == K_DOWN:
            self.moveSelectionDown()
            event.skip()
        if event.key == K_UP:
            self.moveSelectionUp()
            event.skip()
        if event.key == K_RETURN:
            self.DispatchEvent(evtSelection(
                                selection=self.currentSelection[ str(self.widgetCount) ],
                                timetick=event.timetick,
                                source=self)
                              )
            event.skip()
        if event.key == K_ESCAPE:
            self.DispatchEvent(evtQuit(timetick=event.timetick, source=self))
        event.KillEvent()

    def OnMouseMove(self, event):
        newpos = (event.pos[0] - self.rasterPos[0], event.pos[1] - self.rasterPos[1])
        for a in range(len(self.choiceLabels[ str(self.widgetCount) ])):
            if self.choiceLabels[ str(self.widgetCount) ][a].contains(newpos):
                self.currentSelection[ str(self.widgetCount) ] = a

    def OnMouseClick(self, event):
        newpos = (event.pos[0] - self.rasterPos[0], event.pos[1] - self.rasterPos[1])
        for a in range(len(self.choiceLabels[ str(self.widgetCount) ]) ):
            if self.choiceLabels[ str(self.widgetCount) ][a].contains(newpos):
                self.currentSelection[ str(self.widgetCount) ] = a
                self.DispatchEvent(evtSelection(selection=a,
                                               timetick=event.timetick,
                                               source=self) )
        
    def EmptyMenu(self):
        self.choices[ str(self.widgetCount) ] = []
        self.choiceLabels[ str(self.widgetCount) ] = []
            
    def AddChoice(self, **choice):
        if choice.has_key("text"):
            self.choices[ str(self.widgetCount) ].append(choice['text'])
            if choice.has_key("size"):
                theSize = self._getRasterCoordinate( (0, choice['size']) )[1]
            else:
                theSize = self.fontRasterSize
            theChoice = font.mgLabel(
                                        choice['text'],
                                        font=self.font,
                                        pos = (0,0),
                                        size = theSize
                                    )
            self.choiceLabels[ str(self.widgetCount) ].append(theChoice)
            if len(self.choices[ str(self.widgetCount) ]) == 1:
                self.currentSelection[ str(self.widgetCount) ] = 0
            self.masterSurf = graphics.Surface(self.rasterSize)        
        else:
            # Don't add the choice if it's empty
            pass

    def SetChoice(self, choice, text):
        if choice in range(len(self.choices[ str(self.widgetCount) ])):
            self.choices[ str(self.widgetCount) ][choice] = text
            self.choiceLabels[ str(self.widgetCount) ][choice].setText(text)
            if len(self.choices[ str(self.widgetCount) ]) == 1:
                self.currentSelection[ str(self.widgetCount) ] = 0
        else:
            # Don't add the choice if it's empty
            pass
            
    def __len__(self):
        return len(self.choices)
            
    def UpdateView(self, screen, timetick):
        self.masterSurf.fill(self.backgroundColor)
        self.masterSurf.set_alpha(255)
        if self.topRef is not None:
            self.SetRasterPos(self.topRef.GetBottomRasterPos())
        for a in range(len(self.choiceLabels[ str(self.widgetCount) ])):
            self.choiceLabels[ str(self.widgetCount) ][a].setPos( (0, 10 + (a * (self.fontRasterSize + 10) ) ) )
            if a == self.currentSelection[ str(self.widgetCount) ]:
                self.choiceLabels[ str(self.widgetCount) ][a].color = self.fontHighlightColor
                self.choiceLabels[ str(self.widgetCount) ][a].render(self.masterSurf)
            else:
                self.choiceLabels[ str(self.widgetCount) ][a].color = self.fontColor
                self.choiceLabels[ str(self.widgetCount) ][a].render(self.masterSurf)
        screen.blit(self.masterSurf, self.rasterPos)
            
    def moveSelectionDown(self):
        self.currentSelection[ str(self.widgetCount) ] += 1
        if self.currentSelection[ str(self.widgetCount) ] >= len(self.choices[ str(self.widgetCount) ]):
            self.currentSelection[ str(self.widgetCount) ] = len(self.choices[ str(self.widgetCount) ]) - 1

    def moveSelectionUp(self):
        self.currentSelection[ str(self.widgetCount) ] -= 1
        if self.currentSelection[ str(self.widgetCount) ] < 0:
            self.currentSelection[ str(self.widgetCount) ] = 0
            

            