#!/usr/bin/env python

'''

    @version $Id: mg_paged_menu.py,v 1.1 2005/09/27 04:40:44 dave Exp $

    This file is part of pyAlarm.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import pygame
from pygame.locals import *
from libGUIacme import font
from libacme import paths
from libGUIacme.libwidget import *
from libGUIacme.widgets import mg_container
from libGUIacme.widgets import mg_menu
from libGUIacme.widgets import mg_event
from libGUIacme.widgets.mg_constants import *

class mgPagedSelectionEvent(mg_event.mgEvent):
    mgEType = mgEPAGEDSELECTION
    selection = None
    page = None
    
    def __init__(self, **args):
        super(mgPagedSelectionEvent, self).__init__(type=self.mgEType, **args)

        if args.has_key('selection'):
            self.selection = args['selection']
        else:
            raise RuntimeError, 'Paged Selection Event requires a selection!'
        if args.has_key('page'):
            self.page = args['page']
        else:
            raise RuntimeError, 'Paged Selection Event requires a page!'


class mgPagedMenu(mg_container.mgContainer):
    pages = None
    numPages = 0
    topPage = None
    parentPages = None
    
    def __init__(self, **args):
        super(mgPagedMenu, self).__init__(**args)
        self.pages = {}
        self.pages[str(self.widgetCount)] = mg_container.mgContainer()
        self.parentPages = {}
        self.parentPages[str(self.widgetCount)] = {}

    # adds a page to be managed by this container
    def AddPage(self, widget, pageID, parent=None):
        self.AddToPage(widget, pageID)
        if self.topPage == None:
            self.topPage = pageID
        else:
            self.setParentPage(pageID, parent)
        self.numPages += 1

    def setParentPage(self, pageID, parent):
        self.parentPages[str(self.widgetCount)][str(pageID)] = parent

    def getParentPage(self, pageID):
        if pageID != self.topPage:
            return self.parentPages[str(self.widgetCount)][str(pageID)]
        else:
            return None

    def setTopPage(self, pageID):
        self.topPage = pageID

    def OnMenuQuit(self, event):
        if event.sourcewidget.Group() == self.topPage:
            self.DispatchEvent(mg_menu.evtQuit(timetick=event.timetick, source=self) )
        else:
            self.ShowPage( self.getParentPage(event.sourcewidget.Group()) )
        
    def OnMenuSelect(self, event):
        self.DispatchEvent(mgPagedSelectionEvent(selection=event.selection,
                                        page=event.sourcewidget.Group(),
                                        timetick=event.timetick,
                                        source=self) )

    # adds a widget to a page
    def AddToPage(self, widget, pageID):
        widget.SetGroup(pageID)
        if isinstance(widget, mg_menu.mgMenu):
            widget.RegisterHandler(mgESELECTION, self.OnMenuSelect)
            widget.RegisterHandler(mgEQUIT, self.OnMenuQuit)
        self.pages[str(self.widgetCount)].AddWidget(widget, True)
    
    def __len__(self):
        return self.numPages
        
    def ShowPage(self, pageID):
        self.pages[str(self.widgetCount)].ShowGroup(pageID)

    def UpdateView(self, screen, timetick):
        self.pages[str(self.widgetCount)].UpdateView(screen, timetick)

    # Just pass events through to each page
    def processEvent(self, event):
        self.pages[str(self.widgetCount)].processEvent(event)

    def _processEvent(self, event):
        self.processEvent(event)
        
    


