#!/usr/bin/env python

'''

    @version $Id: mg_event.py,v 1.2 2005/09/27 04:40:44 dave Exp $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import pygame
from pygame.locals import *
from libGUIacme import font
from libacme import paths
from libGUIacme.libwidget import *
from libGUIacme.widgets import mg_event
from libGUIacme import graphics
from libGUIacme.widgets import mg_widget
from libGUIacme.widgets.mg_constants import *

class mgTextBox(mg_widget.mgWidgetBase):
    dirty = True
    focused = False
    emptyImage = None
    
    def __init__(self, **args):
        super(mgTextBox, self).__init__(**args)


        if args.has_key('width'):
            width = args['width']
        else:
            width = 80

        if args.has_key('font_color'):
            self.fontColor = args['font_color']
        else:
            self.fontColor = (0,0,0)
        if args.has_key('shadow_color'):
            self.shadowColor = args['shadow_color']
        else:
            self.shadowColor = (200,200,200, 255)
        self.font = font.mgFont(self.fontName, size=self.fontSize, shadow_color=self.shadowColor)
        linesize = self.font.get_linesize()
        self.rect = pygame.Rect( (0,0,width, linesize +4) )
        self.SetPos(self.pos)

        self.emptyImage = graphics.Surface((width, linesize +4))
        self.DrawBase()
        self.image = graphics.Surface((width, linesize +4))
        self.image.blit(self.emptyImage, (0,0) )

        self.text = ''
        self.textPos = (2, 2)

        self.RegisterHandler(mgEMOUSECLICK, self.Click, None)
        self.RegisterHandler(mgEKEYBOARD, self.OnType, None)

    def DrawBase(self):
        self.emptyImage.fill( (0,0,0) )
        pygame.draw.polygon( self.emptyImage, (200,200,200, 255),
                        [ (1,1), (1, self.rect.height-2),
                          (self.rect.width-2, self.rect.height-2), (self.rect.width-2, 1),
                          (1,1) ] )
                         
        if self.focused is True:
            pygame.draw.lines( self.emptyImage, (255,255,255), False,
                                [
                                    (0, self.rect.height-1),
                                    (self.rect.width-1, self.rect.height-1),
                                    (self.rect.width-1, 0)
                                ],
                                1)
        else:
            pygame.draw.lines( self.emptyImage, (255,255,255), False,
                                [
                                    (0, self.rect.height-1),
                                    (0,0),
                                    (self.rect.width-1, 0)
                                ],
                                1)
        
    def update(self, timetick):
        if not self.dirty:
            return

        text = self.text
        if self.focused:
                text += '_'

        textImg = self.font.fprint( text, self.fontColor )
        tmpRect = textImg.get_rect()
        if tmpRect.width > self.rect.width and self.focused:
            txtPos = (self.textPos[0] - (tmpRect.width - self.rect.width) ,
                      self.textPos[1])
        else:
            txtPos = self.textPos
        self.image.blit( self.emptyImage, (0,0) )
        self.image.blit( textImg, txtPos )

        self.dirty = 0

    def UpdateView(self, screen):
        screen.blit(self.image, self.rect)

    #----------------------------------------------------------------------
    def Click(self, event):
        if self.rect.collidepoint( event.pos ):
            self.focused = True
            self.dirty = True
        else:
            if self.focused:
                self.focused = False
                self.dirty = True
        self.DrawBase()

    #----------------------------------------------------------------------
    def SetText(self, newText):
        self.text = newText
        self.dirty = 1

    def GetText(self):
        return self.text

    def OnType(self, event):
        if self.focused:
            if event.key == K_BACKSPACE:
                #strip of last character
                newText = self.text[:( len(self.text) - 1 )]
                self.SetText( newText )
            elif event.key == K_RETURN:
                self.focused = False
                self.dirty = True
                self.DrawBase()
            else:
                newText = self.text + event.character
                self.SetText( newText )



            