#!/usr/bin/env python

'''

    @version $Id: mg_widget.py,v 1.7 2005/10/06 06:30:43 dave Exp $

    This file is part of mathguy.

    mathguy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    mathguy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mathguy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    mathguy is copyright 2005 by David Fancella

'''

import pygame
from pygame.locals import *
from libGUIacme import font
from libacme import paths
from libGUIacme.libwidget import *
from libGUIacme.widgets.mg_constants import *
import libGUIacme.event_manager
from libGUIacme import graphics

mgWIDGETCOUNT = 0
            
# base widget class
class mgWidgetBase(pygame.sprite.Sprite):
    # widgetCount is used internally, don't mess with it
    widgetCount = 0
    
    # The font members all need reasonable defaults, because if a widget doesn't
    # provide any of them, they need to be loaded properly
    # maybe later we'll make it a setting or something
    font = None
    fontHighlight = None
    fontName = None
    fontSize = 100
    fontColor = (255,255,255)
    fontHighlightColor = (40,40,250)
    fontShadow = 10
    size = (500,500)
    pos = (0,0)
    backgroundColor = None
    backgroundImage = None
    
    eventMaster = None
    
    def __init__(self, **args):
        pygame.sprite.Sprite.__init__(self)
        global mgWIDGETCOUNT
        
        mgWIDGETCOUNT += 1
        self.widgetCount = mgWIDGETCOUNT
        self.group = {}
        # housekeeping, we need to have a module name for each widget
        if args.has_key("module)"):
            self.module = args['module']
        if args.has_key("eventmanager"):
            self.eventMaster = args['eventmanager']
        else:
            self.eventMaster = libGUIacme.event_manager.eventMaster
        # figure out the main font for the window and load it
        if args.has_key("font"):
            self.fontName = args["font"]
        if args.has_key("font_size"):
            self.fontSize = args['font_size']
        if args.has_key("font_color"):
            self.fontColor = args['font_color']
        if args.has_key("font_shadow"):
            self.fontShadow = args['font_shadow']
        self.font = font.mgFont(self.fontName, size=self.fontSize)
        
        if args.has_key("size"):
            self.size = args['size']
        if args.has_key("background_color"):
            self.backgroundColor = args['background_color']
        if args.has_key("background_image"):
            self.backgroundImage = args['background_image']
        if args.has_key("pos"):
            self.rect = graphics.Rect( (args['pos'][0], args['pos'][1], 1,1 ) )
            self.widgetPos = args["pos"]
            self.pos = self.widgetPos
        
        self.image = graphics.Surface( (self.size) )
        self.image.convert()
        self.rect = self.image.get_rect()
        self.SetPos(self.pos)

        self.bevelSize = 2

    def DrawBackground(self):
        self.image.blit(self.background, (0,0))

    def DrawAlternateDecoration(self):
        pygame.draw.lines(self.image, (150,150,150), False,
                          [ (self.rect.width - self.bevelSize, 0), (0,0), (0, self.rect.height - self.bevelSize) ],
                          self.bevelSize)
        pygame.draw.lines(self.image, (0,0,0), False,
                          [ (self.rect.width - self.bevelSize, 1),
                            (self.rect.width - self.bevelSize,self.rect.height - self.bevelSize),
                            (1, self.rect.height - self.bevelSize) ],
                          self.bevelSize)
        
    def DrawDecoration(self):
        pygame.draw.lines(self.image, (0,0,0), False,
                          [ (self.rect.width - self.bevelSize, 0), (0,0), (0, self.rect.height - self.bevelSize) ],
                          self.bevelSize)
        pygame.draw.lines(self.image, (150,150,150), False,
                          [ (self.rect.width - self.bevelSize, 1),
                            (self.rect.width - self.bevelSize,self.rect.height - self.bevelSize),
                            (1, self.rect.height - self.bevelSize) ],
                          self.bevelSize)
    
    def RegisterHandler(self, eventType, eventHandler, source):
        self.eventMaster.RegisterListener( source, eventHandler, eventType )

    def GetPos(self):
        return self.rect.topleft

    def SetPos(self, pos):
        self.rect.topleft = pos
        self.pos = pos

    def update(self, timetick):
        raise NotImplemented, 'Update not implemented, this is a bug!'

    def UpdateView(self, screen):
        screen.blit(self.image, self.rect)
    
    def DispatchEvent(self, event):
        self.eventMaster.Post(event)
                


