#!/usr/bin/env python

'''

    @version $Id: mg_event.py,v 1.2 2005/09/27 04:40:44 dave Exp $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import math

# this class contains the map

class aaMap(object):
    mapName = None
    mapAuthor = None
    mapVersion = None
    spawnPoints = []
    walls = []
    obwalls = []
    isDirty = None
    
    def __init__(self):
        self.mapName = ''
        self.mapAuthor = ''
        self.mapVersion = ''
        self.isDirty = True

    def SetName(self, newName):
        self.mapName = newName

    def SetAuthor(self, newAuthor):
        self.mapAuthor = newAuthor

    def SetVersion(self, newVersion):
        self.mapVersion = newVersion

    def AddSpawn(self, **args):
        self.isDirty = True
        # as in the game, angles override vectors
        # since we will be writing all spawn points with angles,
        #  we throw away the vector
        if args['vector'][1] != 0.0:
            vectorAngle = math.atan(args['vector'][0] / args['vector'][1])
        else:
            if args['vector'][0] < 0:
                vectorAngle = math.radians(270)
            elif args['vector'][0] > 0:
                vectorAngle = math.radians(90)
            else:
                vectorAngle = 0.0
        if args['angle'] == None:
            angle = vectorAngle
        else:
            angle = math.radians(args['angle'])

        vectorAngle = round(vectorAngle, 5)
                
        # determine if the angle given is the same angle pointed to by vector
        # This code is broke
        if round(angle,5) != vectorAngle and vectorAngle != 0.0 and angle != 0.0:
            print "Warning: angle is not the same angle that vector points to"
            print angle, math.atan(vectorAngle)

        # ignore it, of course, and only store the angle--in radians
        self.spawnPoints.append( { 'angle' : angle,
                                   'pos' : args['pos'] } )

    def AddWall(self, **args):
        self.isDirty = True
        points = args['points']
        self.walls.append(points)

    def AddObstacleWall(self, **args):
        points = args['points']
        self.obwalls.append(points)

    def SaveMap(self, filename):
        theFile = open(filename, 'w')

        theFile.write('<?xml version="1.0" encoding="ISO-8859-1" standalone="no"?>\n')
        theFile.write('<!DOCTYPE World SYSTEM "map-0.1.dtd">\n')
        theFile.write('<World version="0.1">')
        theFile.write('\n')
        theFile.write('<Map name="' + self.mapName + '" version="' + self.mapVersion + '" author="' + self.mapAuthor + '">\n')
        theFile.write('    <Field>\n        <Axes number="4"/>\n')

        for sp in self.spawnPoints:
            sptxt = '<Spawn x="' + str(sp['pos'][0]) + '" y="' + str(sp['pos'][1]) + '" xdir="1" ydir="0" />\n'
            theFile.write(sptxt)

        for sp in self.walls:
            print sp
            theFile.write('<Wall>\n')
            for p in sp:
                sptxt = '<Point x="' + str(p[0]) + '" y="' + str(p[1]) + '" />\n'
                theFile.write(sptxt)
            theFile.write('</Wall>\n')

        theFile.write('    </Field>\n</Map>\n</World>')
        theFile.close()


