#!/usr/bin/env python

'''

    @version $Id: mg_event.py,v 1.2 2005/09/27 04:40:44 dave Exp $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

# This file parses the map

from xml.sax.handler import ContentHandler
from xml.sax.handler import ErrorHandler
from xml.sax.handler import EntityResolver
import xml
from libacme.map import aaMap
from libacme import paths

import math

# This is the only function you should call externally, it will return a data
# structure that contains the map read.
def ParseMap(theFile):
    contentParser = aaContent()
    errorHandler = aaError()
    theParser = xml.sax.make_parser()
    theParser.setContentHandler(contentParser)
    theParser.setEntityResolver( aaEntResolver() )
    #theParser.setDTDHandler(None)
    theParser.parse(theFile)
    return contentParser.themap

class aaEntResolver(EntityResolver):
    def resolveEntity(self, pubid, sysid):
        return paths.Join('dtd', sysid)
    
# The error handler class, empty to ignore errors for now
class aaError(ErrorHandler):
    def error(self, exc):
        pass

    def fatalError(self, exc):
        pass

    def warning(self, exc):
        pass
    
class aaContent(ContentHandler):
    themap = None
    mymode = 'top'
    currentTag = None
    points = []
    
    def __init__(self):
        self.themap = aaMap()
    
    def startElement(self,name, attrs):
        # Have to check this before checking the mode
        # This is the core part of the parser
        if self.mymode == 'field':
            if self.currentTag == 'wall':
                if name.lower() == "point":
                    self.points.append( (float(attrs.getValue('x')), float(attrs.getValue('y')) ) )
            if self.currentTag == 'obstaclewall':
                if name.lower() == "point":
                    self.points.append( (float(attrs.getValue('x')), float(attrs.getValue('y')) ) )
            if name.lower() == 'spawn':
                theNames = attrs.getNames()
                for a in range(len(theNames)):
                    theNames[a] = theNames[a].lower()
                if 'angle' in theNames:
                    angle = float(attrs.getValue('angle'))
                else:
                    angle = None
                if 'xdir' in theNames:
                    # assume that if xdir is there, ydir is also there
                    vector = (float(attrs.getValue('xdir')), float(attrs.getValue('ydir')))
                else:
                    vector = (0,0)
                self.themap.AddSpawn( pos=(float(attrs.getValue('x')), float(attrs.getValue('y'))),
                                      vector=vector,
                                      angle=angle)
            if name.lower() == 'wall':
                self.currentTag = 'wall'
            if name.lower() == 'obstaclewall':
                self.currentTag = 'obstaclewall'
                
                
        # Now check to see if it's an alarm, or something else
        if name.lower() == "map":
            self.mymode = 'map'
            self.themap.SetName(attrs.getValue('name'))
            self.themap.SetAuthor(attrs.getValue('author'))
            self.themap.SetVersion(attrs.getValue('version'))
        elif name.lower() == 'field':
            self.mymode = 'field'
            
        
    def endElement(self,name):
        if name.lower() == 'wall':
            self.themap.AddWall(points=self.points)
            self.points = []
            self.currentTag = ''
        elif name.lower() == 'obstaclewall':
            self.themap.AddObstacleWall(points=self.points)
            self.points = []
            self.currentTag = ''
                
    def characters(self,content):
        pass

    def processingInstruction(self, target, data):
        print target, data
        
    def GetMap(self):
        self.themap.isDirty = True
        return self.themap




